/*
 * ----------------------------------------------------------------------------
 * RndMp3Copy
 * Licensed under the (Creative Commons) GNU General Public License
 * http://creativecommons.org/licenses/GPL/2.0/
 *
 * Author: Adrian Holfter <aholfter@googlemail.com>
 * ----------------------------------------------------------------------------
 */

#include <iostream>
#include <libglademm.h>
#include <gtkmm.h>

// Activate this if you have a windows machine
//#define __WINDOWS__

using namespace std;

Gtk::Window* pMainWindow = 0;
Gtk::Button* pBtnGo = 0;
Gtk::SpinButton* pSpinButton = 0;
Gtk::FileChooserButton* pInput = 0;
Gtk::FileChooserButton* pOutput = 0;
Gtk::FileChooserButton* pUsedList = 0;
Gtk::CheckButton* pCmp3 = 0;
Gtk::CheckButton* pCwma = 0;
Gtk::CheckButton* pCogg = 0;
Gtk::CheckButton* pCaac = 0;
Gtk::CheckButton* pCflac = 0;
Gtk::CheckButton* pCinExp = 0;
Gtk::CheckButton* pCexExp = 0;
Gtk::CheckButton* pCusedList = 0;
Gtk::RadioButton* pRformat0 = 0;
Gtk::RadioButton* pRformat1 = 0;
Gtk::RadioButton* pRformat2 = 0;
Gtk::Entry* pIinExp = 0;
Gtk::Entry* pIexExp = 0;

void on_btn_go_clicked(){
    // Launch RndMp3Copy
    string filters = "";
    if(pCmp3->get_active()){ filters += "mp3,"; }
    if(pCwma->get_active()){ filters += "wma,"; }
    if(pCogg->get_active()){ filters += "ogg,"; }
    if(pCaac->get_active()){ filters += "aac,"; }
    if(pCflac->get_active()){ filters += "flac,"; }
    if(filters.length() > 0){
        filters.erase(filters.length() -1, 1);
    }

    string mode = "0";
    if(pRformat0->get_active()){
        mode = "0";
    }else if(pRformat1->get_active()){
        mode = "1";
    }else if(pRformat2->get_active()){
        mode = "2";
    }

    char buf[256];
    sprintf(buf,"%f",pSpinButton->get_value());

    //string cmdline;
    vector<string> cmdline;
    #ifdef __WINDOWS__
        cmdline.push_back("RndMp3Copy.exe");
    #else
        cmdline.push_back("/usr/bin/xterm");
        cmdline.push_back("-e");
        cmdline.push_back("./RndMp3Copy");
        cmdline.push_back("-w");
    #endif

    cmdline.push_back("-i");
    cmdline.push_back(pInput->get_filename());
    cmdline.push_back("-o");
    cmdline.push_back(pOutput->get_filename());
    cmdline.push_back("-s");
    cmdline.push_back(buf);
    cmdline.push_back("-d");
    cmdline.push_back(mode);
    if(filters.length() > 0){
        cmdline.push_back("-f");
        cmdline.push_back(filters);
    }
    if(pCinExp->get_active()){
        cmdline.push_back("-in");
        cmdline.push_back(pIinExp->get_text());
    }
    if(pCexExp->get_active()){
        cmdline.push_back("-ex");
        cmdline.push_back(pIexExp->get_text());
    }
    if(pCusedList->get_active()){
        cmdline.push_back("-l");
        cmdline.push_back(pUsedList->get_filename());
    }

    cout << "cmdline is: ";
    for(unsigned int i=0; i<cmdline.size(); i++){
        cout << cmdline[i] << " ";
    }
    cout << endl;

    Glib::spawn_async(".", cmdline,Glib::SpawnFlags(0),sigc::slot<void>(),NULL);

    return;

}

int main (int argc, char **argv){
    Gtk::Main kit(argc, argv);

    //Load the Glade file and instiate its widgets:
    Glib::RefPtr<Gnome::Glade::Xml> refXml;
    #ifdef GLIBMM_EXCEPTIONS_ENABLED
    try{
        refXml = Gnome::Glade::Xml::create("RndMp3CopyGui.glade");
    }catch(const Gnome::Glade::XmlError& ex){
        cerr << ex.what() << endl;
        Gtk::MessageDialog dialog ("Error creating Main Window",false,Gtk::MESSAGE_INFO,Gtk::BUTTONS_OK,false);
        dialog.set_secondary_text(ex.what());
        dialog.run();
        return 1;
    }
    #else
    std::auto_ptr<Gnome::Glade::XmlError> error;
    refXml = Gnome::Glade::Xml::create("RndMp3CopyGui.glade", "", "", error);
    if(error.get()){
        cerr << error->what() << endl;
        Gtk::MessageDialog dialog ("Error creating Main Window",false,Gtk::MESSAGE_INFO,Gtk::BUTTONS_OK,false);
        dialog.set_secondary_text(ex.what());
        dialog.run();
        return 1;
    }
    #endif

    //Get the Glade-instantiated Dialog:
    refXml->get_widget("MainWindow", pMainWindow);
    if(!pMainWindow){
        cerr << "No Main Window found!" << endl;
        Gtk::MessageDialog dialog ("Error creating Main Window",false,Gtk::MESSAGE_INFO,Gtk::BUTTONS_OK,false);
        dialog.set_secondary_text("The Glade-File is probably messed up.");
        dialog.run();
        return -1;
    }

    refXml->get_widget("btn_go", pBtnGo);
    if(pBtnGo){
        pBtnGo->signal_clicked().connect( sigc::ptr_fun(&on_btn_go_clicked) );
    }

    refXml->get_widget("spinbutton1", pSpinButton);
    refXml->get_widget("btn_source", pInput);
    refXml->get_widget("btn_target", pOutput);
    refXml->get_widget("btn_usedlist", pUsedList);
    refXml->get_widget("chk_mp3", pCmp3);
    refXml->get_widget("chk_wma", pCwma);
    refXml->get_widget("chk_ogg", pCogg);
    refXml->get_widget("chk_aac", pCaac);
    refXml->get_widget("chk_flac", pCflac);
    refXml->get_widget("chk_in", pCinExp);
    refXml->get_widget("chk_ex", pCexExp);
    refXml->get_widget("chk_usedlist", pCusedList);
    refXml->get_widget("RadioFormat0", pRformat0);
    refXml->get_widget("RadioFormat1", pRformat1);
    refXml->get_widget("RadioFormat2", pRformat2);
    refXml->get_widget("in_in", pIinExp);
    refXml->get_widget("in_ex", pIexExp);

    // Enter Main Loop
    kit.run(*pMainWindow);

    delete pMainWindow;
    pMainWindow = 0;

    return 0;
}
